//#include "global.h"

#include "custom_port.h"
#include <arduino.h>
//#include <arpschuino32_core.h>
#include <arpschuino.h>
#include "port.h"
#include "I2C_port.h"
#include <vector>
using namespace std;


//put here the library you want to use

#ifdef PW_ENABLE
  extern Port portW;
#endif
extern Port portA;
#ifdef PB_ENABLE
  extern Port portB;
#endif
extern vector<Device*>I2C_port ;


#ifdef PW_ENABLE
//////////// port W (wilulu out) ////////////// 
const uint8_t button = Arp[0];//PIN for the button to select DMX or potentiometer
const uint8_t pot = Arp[1];//PIN for the potentiometer
uint16_t actual_level;
uint16_t target_level;


void setupW()//this loop is executed once at startup
{
  portW.setNbDmxChannels(1);//the number of channels used in this port. REQUIRED !
  ledcSetup(8, 2441, 15); // configure LED PWM functionalitites
  ledcAttachPin(WiluluOUT_0, 8);     // attach the channel to the GPIO to be controlled 

  pinMode(button, INPUT_PULLUP);
  pinMode(pot, INPUT);
} 
void action_W(const std::vector<uint8_t> & DMXslice)  //this loop is executed at each signal reception
{
  if(digitalRead(button))
  {
    target_level = (((DMXslice[0] + 1) * (DMXslice[0] + 1) - 1) >> 1); // level = (((DMXslice[i]+1)*(DMXslice[i]+1)-1)>>(16-m_pwmRes));
  }
  else
  {
    uint16_t temp = analogRead(pot);
    target_level = (((temp + 1) * (temp + 1) - 1) >> 9); // temp should be a 12 bits value
    // or :  target_level = analogRead(pot)<<3;//no curve
  }
}
void at_each_loop_W() 
{
    int32_t level;
    if (actual_level < target_level)
    {
        level = actual_level + 1 + (actual_level / 256); // m_actual_level[i]++;
        if (level > target_level)
            actual_level = target_level;
        else
            actual_level = level;
        ledcWrite(8, actual_level);
    }
    else if (actual_level > target_level)
    {
        level = actual_level - 1 - (actual_level / 256); // m_actual_level[i]--;
        if (level < target_level)
            actual_level = target_level;
        else
            actual_level = level;
        ledcWrite(8, actual_level);
    }
}
#endif


//////////// port A //////////////
#include <ArpAccelStepper.h>

ArpAccelStepper stepper(ArpAccelStepper::DRIVER, 22, 21);

void setupA()   //this loop is executed once at startup
{   
  portA.setNbDmxChannels(4);
  stepper.setMinPulseWidth(7);
  stepper.setMaxSpeed(10000.0);
  stepper.setAcceleration(4000.0);
  //stepper.moveTo(32000);//20 tours à 1600 stp/tour
}

void action_A(const std::vector<uint8_t> & DMXslice)  //this loop is executed at each signal reception
{
  //stepper.moveTo(map(DMXslice[0],0,255,0,32000));
  //stepper.move(10000);
  stepper.setSpeed(DMXslice[0]*20);

}

void at_each_loop_A()   //this loop is executed continuously
{
  stepper.runSpeed();
  //stepper.setSpeed(10000);
}


#ifdef PB_ENABLE  ///////////////////////////////////////////////
//////////// port B //////////////
//read MCP9700 temp sensor
#include <OneWire.h>
#include <DallasTemperature.h>


// const uint8_t sensorPin = Arp12;
const long interval = 500;
unsigned long previousMillis = 0;        // will store last time LED was updated

const uint8_t LED = 8;
bool dmxRes;
uint8_t pwmRes;
uint16_t target_level;
uint16_t actual_level;

OneWire oneWire (Arp10);//(Arp9);
DallasTemperature DS18B20(&oneWire);
float tempC; // temperature in Celsius 
const uint8_t fan = 12;//25kHz pwm

bool safe_mode = false;

void setupB()      //this loop is executed once at startup
{
  portB.setNbDmxChannels(2);//2???//the number of channels used in this port. REQUIRED !
  
  //sensor :
  DS18B20.begin();    // initialize the DS18B20 sensor

  //fan :
  ledcSetup(fan, 25000, 8); // 25kHz 8 bits
  ledcAttachPin(Arp[fan], fan);     // attach the channel to the GPIO to be controlled

  //LED :
  int32_t freq = 40000000;
  pwmRes = 13;
  for (uint8_t i = 1; i < pwmRes; i++)
  {
      freq = freq / 2;
  }
  dmxRes = true; //false 8 bits, true 16 bits

  ledcSetup(LED, freq, pwmRes); // configure LED PWM functionalitites
  ledcAttachPin(Arp[LED], LED);     // attach the channel to the GPIO to be controlled



  Serial.println(": custom, setup done :");
  //Serial.println("read MCP9700 temp sensor");
  Serial.println("DS18B20 temp sensor");
  Serial.print("PWM ");
  Serial.print(pwmRes);
  Serial.print(" bits; freq ");
  Serial.print(freq);
  Serial.print(" hz; ");
  if (dmxRes)
      Serial.print(", 16 bits DMX input");
  Serial.println();  
}

void action_B(const std::vector<uint8_t> & DMXslice)    //this loop is executed at each signal reception
{  
    if (dmxRes)
    { // if 16 bits DMX
        target_level = DMXslice[0 * 2] << 8 | DMXslice[0 * 2 + 1];             // level= DMXslice[i*2] << 8 | DMXslice[i*2+1];
        uint32_t level32 = ((target_level + 1) * (target_level + 1)) - 1; // uint32_t level32 =((level+1)*(level+1))-1;
        target_level = level32 >> (32 - pwmRes);                             // level = level32 >>(32-m_pwmRes);
    }
    else
    { // if 8 bits DMX
        target_level = (((DMXslice[0] + 1) * (DMXslice[0] + 1) - 1) >> (16 - pwmRes)); // level = (((DMXslice[i]+1)*(DMXslice[i]+1)-1)>>(16-m_pwmRes));
    }

    if(safe_mode) target_level = target_level/10;//2;
}

void at_each_loop_B()   //this loop is executed continuously
{

                int32_t level;
                if (actual_level < target_level)
                {
                    level = actual_level + 1 + (actual_level / 256); // m_actual_level[i]++;
                    if (level > target_level)
                        actual_level = target_level;
                    else
                        actual_level = level;
                    ledcWrite(LED, actual_level);
                }
                else if (actual_level > target_level)
                {
                    level = actual_level - 1 - (actual_level / 256); // m_actual_level[i]--;
                    if (level < target_level)
                        actual_level = target_level;
                    else
                        actual_level = level;
                    ledcWrite(LED, actual_level);
                }



  unsigned long currentMillis = millis(); 
  if (currentMillis - previousMillis >= interval) { //fan controll
    previousMillis = currentMillis;  

    DS18B20.requestTemperatures();       // send the command to get temperatures
    tempC = DS18B20.getTempCByIndex(0);  // read temperature in °C  
    Serial.print(tempC);    // print the temperature in °C
    Serial.print("°C");   

    uint16_t pwmFan;

    if(tempC ==-127.00) 
    {
      safe_mode =true;
      pwmFan = 255;
    }   

    else
    {
      if(tempC<= 40)
      {
        pwmFan = 0;
        safe_mode =false;
      }        
      else if(tempC >110)
      {
        //pwmFan = 255;
        safe_mode =true;
      }    
      else 
      {
        pwmFan = tempC*2.5;//*3   //Operating temperature range: -55ºC to +125ºC
        safe_mode =false;
      }
    }

    Serial.print(" , pwmFan : "); 
    Serial.print(pwmFan);

    Serial.print(" , Mode : "); 
    if(safe_mode)Serial.print("safe");
    else Serial.print("normal");

    Serial.print("target_level : ");
    Serial.print(target_level);
    Serial.print(" : ");
    Serial.print(target_level>>5);


    Serial.println();    
 
  
    ledcWrite(fan, pwmFan);

  }     
}
#endif
//////////// port I2C //////////////


void setupI2C(uint8_t ID )   //this loop is executed once at startup
{
  setup();
    switch(ID)
    {
      case 0 : {
        I2C_port[ID]->setNbDmxChannels(8);//the number of channels used by this device. REQUIRED !
        //put your setup for ID0 device here
          break;
      }
      case 1 : {
        I2C_port[ID]->setNbDmxChannels(8);//the number of channels used by this device. REQUIRED !
        //put your setup for ID1 device here          
          break;
      } 
      case 2 : {
        I2C_port[ID]->setNbDmxChannels(8);//the number of channels used by this device. REQUIRED !
        //put your setup for ID0 device here          
          break;
      } 
      //etc, 6 devices max
    }                   
}

void actionI2C(uint8_t ID,const std::vector<uint8_t> & DMXslice)  //this loop is executed at each signal reception
{
  switch(ID)
  {
    case 0 : {//ID0 device
      
      break;
    }
    case 1 : {//ID1 device
      
      break;
    } 
    case 2 : {//ID2 device
      
      break;
    } 
  } 
}

void at_each_loop_I2C(uint8_t ID)   //this loop is executed continuously
{
  switch(ID)
  {
    case 0 : {//ID0 device
      
      break;
    }
    case 1 : {//ID1 device
      
      break;
    } 
    case 2 : {//ID2 device
      
      break;
    }
  }  
}